/***************************************************************************
 *   Copyright (C) 2008  Antony Martin <martin(dot)antony(at)yahoo(dot)fr> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 11/07/2008 */

#ifndef SCESCENE_H
#define SCESCENE_H

#include <SCE/interface/SCEState.h>
#include <SCE/interface/SCECamera.h>
#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCEModel.h>

#ifdef __cplusplus
extern "C"
{
#endif

/* NOTE: doit etre superieur ou egal a SCE_MAX_SHADER_LIGHTS sinon c'est con */
#define SCE_MAX_LIGHTS_ON_MODEL 16

/* nombre maximums de rendus "imbriques" qu'on peut faire */
#define SCE_MAX_RENDER_INFOS 8

/**
 * \ingroup scene
 * @{
 */

/** 
 * \brief "Foreach" callback function type for scene's tree walking
 * \param model the current model to work on
 * \param param an user-defined parameter
 * \returns SCE_TRUE to continue to walk through scene's model tree or SCE_FALSE
 *          to stop walking.
 * 
 * This type of function is used as a callback to work on each scene's model one
 * by one. It is called for each scene's model until it return SCE_FALSE or the
 * model tree ends.
 * 
 * \see SCE_Scene_ForEachModel()
 * \see SCE_Scene_ForEachSelectedModel()
 */
typedef int (*SCE_FSceneForEachModelFunc)(SCE_SModel *model, void *param);

/* etats actives pour une scene */
/** \copydoc sce_sscenerenderstates */
typedef struct sce_sscenerenderstates SCE_SSceneRenderStates;
/**
 * \brief State of a scene's renderer
 */
struct sce_sscenerenderstates
{
    int frustum_culling; /**< Use or not the frustum culling */
    int use_skybox;      /**< Use or not a skybox */
    int lighting;        /**< Use or not lighting */
    int lod;             /**< Use or not LOD for scene's models */
};

/* informations ponctuelles sur une scene (son etat de rendu par ex.) */
/** \copydoc sce_sscenemodelrenderinfos */
typedef struct sce_sscenemodelrenderinfos SCE_SSceneModelRenderInfos;
/**
 * \brief Scene model's renderer infos
 * \todo doc: description
 */
struct sce_sscenemodelrenderinfos
{
    SCE_SModel *model;                           /**< A SCE_SModel */
    int draw;                                    /**< Draw or not the model */
    /* lumieres a appliquer sur le modele */
    SCE_SLight *lights[SCE_MAX_LIGHTS_ON_MODEL]; /**< Lights affecting the
                                                  *   model */
    int num_lights;                              /**< Number of light affecting
                                                  *   the model */
};

/* informations ponctuelles sur une scene (son etat de rendu par ex.) */
/** \copydoc sce_sscenerenderinfos */
typedef struct sce_sscenerenderinfos SCE_SSceneRenderInfos;
/**
 * \brief Scene's renderer infos
 * \todo doc: description
 */
struct sce_sscenerenderinfos
{
    SCE_STexture *rendertarget;     /**< Scene's render target */
    SCEuint cubeface;               /**< Face of the cubemap (used if
                                     *   sce_sscenerenderinfos.rendertarget is
                                     *   a cubemap) */
    SCE_SCamera *camera;            /**< The camera for the render */
    SCE_SList *models;              /**< Models to render */
    SCE_SSceneRenderStates states;  /**< Scene's state */
};

/* scene */
/** \copydoc sce_sscene */
typedef struct sce_sscene SCE_SScene;
/**
 * \brief Scene desctiption structure
 */
struct sce_sscene
{
    SCE_SNode *rootnode;     /**< Root node */

    SCE_SList *models;       /**< Scene's models list */
    SCE_SList *lights;       /**< Scene's lights list */
    int num_models;          /**< Total number of models */

    /* informations bruts pour un rendu */
    SCE_SSceneRenderInfos render[SCE_MAX_RENDER_INFOS]; /**< Array of raw
                                                         *   render infos */
    int ri;                  /**< current render infos ID */
};

/** @} */

/* initialise le gestionnaire de scenes */
int SCE_Init_Scene (void);
/* quitte le gestionnaire de scenes */
void SCE_Quit_Scene (void);

/* cree une nouvelle scene */
SCE_SScene* SCE_Scene_Create (void);
/* supprime une scene */
void SCE_Scene_Delete (SCE_SScene*);

/* genere les boundings boxes de tous les modeles de la scene */
void SCE_Scene_GenerateBoundingsBoxes (SCE_SScene*);

/* retourne le noeud racine */
SCE_SNode* SCE_Scene_GetRootNode (SCE_SScene*);

/* ajoute un noeud au noeud racine */
int SCE_Scene_AddNode (SCE_SScene*, SCE_SNode*);
/* ajoute un modele a une scene */
int SCE_Scene_AddModel (SCE_SScene*, SCE_SModel*);
/* ajoute une skybox a une scene */
int SCE_Scene_AddSkybox (SCE_SScene*, SCE_SModel*);
/* ajoute une lumiere a une scene */
int SCE_Scene_AddLight (SCE_SScene*, SCE_SLight*);

/* execute une fonction pour chaque modele de la scene */
void SCE_Scene_ForEachModel (SCE_SScene*, SCE_FSceneForEachModelFunc, void*);
/* execute une fonction pour chaque modele selectionne pour le rendu */
void SCE_Scene_ForEachSelectedModel (SCE_SScene*, SCE_FSceneForEachModelFunc,
                                     void*);

/* callback de rendu par defaut d'une skybox */
void SCE_Scene_RenderSkybox (SCE_SModel*, void*);

/* prepare la scene pour le rendu */
void SCE_Scene_PrepareRendering (SCE_SScene*);

/* push une update dans la liste */
void SCE_Scene_PushUpdate (SCE_SScene*);
/* pop une update dans la liste */
void SCE_Scene_PopUpdate (SCE_SScene*);

/* met a jour la scene et stocke les informations de rendu dans
   l'element courant de la liste des infos de rendu */
void SCE_Scene_Update (SCE_SScene*, SCE_SCamera*, SCE_STexture*, SCEuint);
/* rend une scene a partir des infos courantes de rendu */
void SCE_Scene_Render (SCE_SScene*, SCE_SCamera*, SCE_STexture*, int);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* guard */
